/* 
 * Copyright (C) 2000 the xine project
 * 
 * This file is part of xine, a unix video player.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: audio_null_out.c,v 1.8 2000/12/10 00:54:32 guenter Exp $
 */

#include <config.h>

#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <math.h>
#include <inttypes.h>

#include "xine.h"
#include "monitor.h"
#include "audio_out.h"
#include "metronom.h"

#define AUDIO_NUM_FRAGMENTS    15
#define AUDIO_FRAGMENT_SIZE  8192

extern uint32_t xine_debug;

static uint32_t sample_rate;   /* in Hz (= samples per second) */
static uint32_t sample_k_rate; /* in kHz */
static uint32_t num_channels;
static uint32_t bytes_in_buffer;
static int audio_triggered;
static struct timeval start_time;

static uint32_t lastPTS;
static uint32_t lastPos;
static int audio_step; /* pts per 32 768 samples (sample = byte/2) */

/* 
 * pts <-> pseudo samples assoc 
 */ 

#define PTS_TAB_NUM 50
static uint32_t audio_pts_tab[PTS_TAB_NUM]; /* organization: ringbuffer */
static uint32_t audio_pts_pos[PTS_TAB_NUM];
static int audio_pts_num, audio_pts_start ;

/*
 * open the "audio device" for writing to
 */
static uint32_t ano_open(uint32_t bits, uint32_t rate, uint32_t channels)
{

  if (sample_rate == (rate*2*channels))
    return 1;

  lastPTS = 0;
  audio_pts_num = 0;
  audio_pts_start = 0;
  num_channels = channels;
  sample_rate = rate * 2 * channels;
  sample_k_rate = rate * 2 * channels / 1000;
  xprintf (VERBOSE|AUDIO, "Pseudo Audio rate : %d bytes/sec (%d kHz)\n", 
	   sample_rate, sample_k_rate/2 / channels);

  audio_step = (uint32_t) 90000 * (uint32_t) 32768 / (rate * channels);
  xprintf (VERBOSE|AUDIO, "audio_null_out : audio_step %d pts per 32768 (pseudo) samples\n",
	   audio_step);
  metronom_set_audio_rate (audio_step);

  bytes_in_buffer = 0;
  gettimeofday(&start_time, NULL);

  return 1;
}


static void ano_put_samples(int16_t* output_samples,
			   uint32_t num_samples, 
			   uint32_t pts_)
{

  uint32_t vpts;

  xprintf (VERBOSE|AUDIO, "Audio : pseudo play %d samples at pts %d pos %d \n",
	   num_samples, pts_, bytes_in_buffer);   

  /*
   * remember vpts
   */

  vpts = metronom_got_audio_samples (pts_, num_samples);

  if (audio_pts_num >= PTS_TAB_NUM) {
      
    audio_pts_tab[audio_pts_start] = vpts;
    audio_pts_pos[audio_pts_start] = bytes_in_buffer;
    
    audio_pts_start = (audio_pts_start + 1) % PTS_TAB_NUM;

  } else {

    audio_pts_tab[audio_pts_num] = vpts;
    audio_pts_pos[audio_pts_num] = bytes_in_buffer;
    audio_pts_num++;

  }


  /*
   * step values
   */

  bytes_in_buffer += num_samples<<1;

  if (!audio_triggered && (bytes_in_buffer > 15000)) {

    xprintf(VERBOSE|AUDIO, "******** PSEUDO AUDIO TRIGGERED ********  bytes_in_buffer: %d\n",
	    bytes_in_buffer);
    audio_triggered = 1;
    gettimeofday(&start_time, NULL);
  }

}

uint32_t get_pseudo_buffer_pos(void) {

  uint32_t pos;

  struct timeval tv;  
  
  if (audio_triggered) {
    gettimeofday(&tv, NULL);
    pos  = (tv.tv_usec + 1000000 - start_time.tv_usec) * sample_k_rate / 1000;
    pos += (tv.tv_sec - start_time.tv_sec) * sample_rate;
    pos -= sample_rate;
    
    xprintf(VERBOSE|AUDIO, "null_audio: Pseudo buffer pos is %d "
	    "(delta: %ld.%ld; %6d Bytes in buffer)\n", 
	    pos, tv.tv_sec - start_time.tv_sec, 
	    tv.tv_usec - start_time.tv_usec, bytes_in_buffer);
    return pos;
  }
  else {
    return 0;
  }
}


static uint32_t ano_get_current_pts (void) {

  int pos;
  int i;
  uint32_t ref_pos1, ref_pos2;
  uint32_t ref_pts1, ref_pts2;
  uint32_t m,pts=0;

  xprintf(VERBOSE|AUDIO, "ano_get_current_pts\n");

  pos = get_pseudo_buffer_pos();

    /*
     * tab-based approach
     */

  if (audio_pts_num>2) {
    i = 1;
    
    do {
      ref_pos2 = audio_pts_pos[ (audio_pts_start+i)%PTS_TAB_NUM ];
      i++;
    } while ((i<(audio_pts_num)) && (ref_pos2<pos)) ;
    i--;
    
    ref_pos1 = audio_pts_pos[ (audio_pts_start+i-1)%PTS_TAB_NUM ];
    
    xprintf (VERBOSE|VPTS, "PTS_TAB: (positions) %d < %d < %d (i=%d)\n",
	    ref_pos1, pos, ref_pos2, i);
    
    if (ref_pos2<ref_pos1) {

      xprintf (VERBOSE|AUDIO, "clock skew detected!\n");

      i--;
      ref_pos2 = audio_pts_pos[ (audio_pts_start+i)%PTS_TAB_NUM ];
      ref_pos1 = audio_pts_pos[ (audio_pts_start+i-1)%PTS_TAB_NUM ];

      xprintf (VERBOSE|VPTS, "=> PTS_TAB: (positions) %d < %d < %d\n", ref_pos1,pos, ref_pos2);
    }

    ref_pts2 = audio_pts_tab [ (audio_pts_start+i)%PTS_TAB_NUM ];
    ref_pts1 = audio_pts_tab [ (audio_pts_start+i-1)%PTS_TAB_NUM ];
    
    xprintf (VERBOSE|VPTS, "PTS_TAB: (pts)       %d < ?? < %d\n", ref_pts1,ref_pts2);
    
      /*
       * linear interpolation
       */
    
    m = (ref_pts2 - ref_pts1) * 32768 / (ref_pos2 - ref_pos1);
    
    xprintf (VERBOSE|VPTS, "PTS_TAB: m=%d\n",m); 
    
    pts = ref_pts1 + (pos - ref_pos1) * m / 32768;
    
    xprintf (VERBOSE|VPTS, "PTS_TAB: => pts = %d (%d sec)\n",pts,pts/90000); 
  }
  /*
    else {
    
    pts = pos * 90000 / sample_rate / num_channels;
    
    xprintf (VERBOSE|VPTS, "PTS_TAB: Not enough data => estimating pts = %d\n", pts);
    }
  */
   else
     xprintf (VERBOSE|VPTS, "PTS_TAB: Not enough data => 0 pts\n");

  /* 
   * check
   */
  
  if (pts < lastPTS) 
    xprintf (VERBOSE|AUDIO, "Ouch! last PTS: %d > %d, last pos: %d (current: %d)\n",
	     lastPTS,pts, lastPos,pos);
  
  lastPTS = pts; 
  lastPos = pos;
  
  return pts;
}


static int ano_get_buf_low (void) {

  int nFill;

  xprintf(VERBOSE|AUDIO, "ano_get_buf_low()\n");

  nFill = bytes_in_buffer - get_pseudo_buffer_pos();

  if (nFill < (AUDIO_NUM_FRAGMENTS * AUDIO_FRAGMENT_SIZE / 8)) {
    xprintf (VERBOSE|AUDIO, "AUDIO HELP - Buffer low %d !\n", nFill);
    return 2;
  }  else if (nFill < (AUDIO_NUM_FRAGMENTS * AUDIO_FRAGMENT_SIZE / 4))
    return 1;
  
  return 0;
}

static void ano_close(void)
{
  audio_triggered = 0;
  sample_rate = 0; /* force reinitialization in next ano_open() */
}


static ao_functions_t audio_nullout = {
  ano_open,
  ano_put_samples,
  ano_get_buf_low,
  ano_get_current_pts,
  ano_close,
};


ao_functions_t *audio_nullout_init () 
{
  audio_triggered = 0;
  ano_open(16, 1000, 2); /* initialize in case we don't play at all */

  return &audio_nullout;
}

