/* 
 * Copyright (C) 2000 the xine project
 * 
 * This file is part of xine, a unix video player.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: video_out.h,v 1.10 2000/12/06 02:36:17 guenter Exp $
 *
 *
 * xine version of video_out.h 
 * handles frame-ordering itself
 *
 * due to performance reasons, there is no wrapper function for
 *
 * notify_frame_area_ready 
 *
 * call this directly using gVideoDriver->notify_frame_area_ready
 *
 * for all other functions, use the wrappers below
 *
 */

#ifndef HAVE_VIDEO_OUT_H

#define HAVE_VIDEO_OUT_H

#include "xine.h"
#include "inttypes.h"
#include <X11/Xlib.h>


typedef struct vo_image_buffer_s /* public part, video drivers may add private fields */
{
  struct vo_image_buffer_s  *next;

  uint32_t                   PTS;
  int                        bFrameBad; /* e.g. frame skipped or based on skipped frame */
  uint8_t                   *mem[3];
  int                        nType;     /* I, B or P frame */

  int                        bDisplayLock, bDecoderLock;
} vo_image_buffer_t;


/*
 * init xine video output, check for usable output drivers
 *
 * result in global variable vo_driver_t *gVideoDriver;
 *
 */

void vo_init (int no_xv) ;

/*
 * set video image size
 * display driver can allocate image buffers now
 *
 *    params : width      == width of video to display.
 *             height     == height of video to display.
 *             ratio      == aspect ration information
 *             rate_code  == mpeg frame rate code
 */

void vo_set_size (uint32_t width, uint32_t height, uint32_t ratio, int rate_code);

/*
 * allocate an image buffer from display driver
 */
vo_image_buffer_t* vo_alloc_image_buffer();

/*
 * image buffer is no longer used by decoder
 */

void vo_free_image_buffer(vo_image_buffer_t *vo_img);

/*
 * notify driver that a certain area of the image is ready for copying
 *
 * return value: 0 => driver ignored this, 1 => driver copied section
 */

int vo_notify_frame_area_ready (vo_image_buffer_t *img, int nLine, int x0, int x1);

/*
 * add frame to display queue
 */

void vo_queue_frame (vo_image_buffer_t* img);

/*
 * set display driver to fullscreen mode
 */

void vo_set_fullscreen (int bFullscreen);

/*
 * returns 1 if driver is in fullscreen mode
 */

int vo_is_fullscreen (void);


/*
 * check if image buffer is full
 */

int vo_is_buffer_full ();

/*
 * check if there is an image to display synced to current audio pts
 */

void vo_video_out ();

/*
 * handle events for video window (e.g. expose)
 */

void vo_handle_event (XEvent *event) ;

/*
 * turn logo mode on/off
 */

void vo_set_logo_mode (int bLogoMode);

/*
 * reset output device (reset buffers etc.)
 */

void vo_reset (void);

/*
 * hide/show mouse cursor.
 */
void vo_display_cursor (int state);

/* get/set aspect ratio */

void vo_set_aspect (int ratio);
int vo_get_aspect ();

/*
 * these are the functions every display driver has to implement:
 *
 * only notify_frame_portion_ready should be called directly.
 */

typedef struct vo_driver_s
{

  int (*get_capabilities) (void); /* see below */

  /* return 1 if image buffers need to be re-allocated */
  int (*set_size)(uint32_t width, uint32_t height, uint32_t ratio);

  vo_image_buffer_t* (*alloc_image_buffer) (void);

  void (*dispose_image_buffer) (vo_image_buffer_t *img);

  void (*process_macroblock) (vo_image_buffer_t *img, 
			      uint8_t *py, uint8_t *pu, uint8_t *pv, /* source */
			      int slice_offset, /* pos in dest frame */
			      int offset);

  void (*display_frame) (vo_image_buffer_t* img);

  void (*set_fullscreen) (int bFullscreen);

  int (*is_fullscreen) (void);

  void (*handle_event) (XEvent *event) ;

  void (*set_logo_mode) (int bLogoMode);

  void (*reset) (void);

  void (*hideshow_cursor) (int state);

  void (*set_aspect) (int ratio);

  int (*get_aspect) (void);

} vo_driver_t;

/* capabilities */

/* driver copies image (i.e. converts it to 
   rgb buffers in the private fields of image buffer) */
#define VO_CAP_COPIES_IMAGE 1

#define ASPECT_AUTO        0
#define ASPECT_FULL        2 /* 4:3  */
#define ASPECT_ANAMORPHIC  1 /* 16:9 */
#define ASPECT_DVB         3 /* 1:2 */

vo_driver_t *init_video_out_xv () ;
vo_driver_t *init_video_out_xshm () ;

/* FIXME: Write output drivers for those too 
   vo_functions_t *init_video_out_x11 () ;
   FIXME
*/

#endif
